@@ -25,7 +25,7 @@
 #ifndef NAUTILUS_FILE_H
 #define NAUTILUS_FILE_H
 
-#include <gtk/gtkobject.h>
+#include <gtk/gtk.h>
 #include <gio/gio.h>
 #include <libnautilus-private/nautilus-file-attributes.h>
 #include <libnautilus-private/nautilus-icon-info.h>
@@ -77,7 +77,9 @@ typedef enum {
 	NAUTILUS_FILE_ICON_FLAGS_IGNORE_VISITING = (1<<1),
 	NAUTILUS_FILE_ICON_FLAGS_EMBEDDING_TEXT = (1<<2),
 	NAUTILUS_FILE_ICON_FLAGS_FOR_DRAG_ACCEPT = (1<<3),
-	NAUTILUS_FILE_ICON_FLAGS_FOR_OPEN_FOLDER = (1<<4)
+	NAUTILUS_FILE_ICON_FLAGS_FOR_OPEN_FOLDER = (1<<4),
+	/* whether the thumbnail size must match the display icon size */
+	NAUTILUS_FILE_ICON_FLAGS_FORCE_THUMBNAIL_SIZE = (1<<5)
 } NautilusFileIconFlags;	
 
 /* Emblems sometimes displayed for NautilusFiles. Do not localize. */ 
@@ -87,6 +89,7 @@ typedef enum {
 #define NAUTILUS_FILE_EMBLEM_NAME_TRASH "trash"
 #define NAUTILUS_FILE_EMBLEM_NAME_NOTE "note"
 #define NAUTILUS_FILE_EMBLEM_NAME_DESKTOP "desktop"
+#define NAUTILUS_FILE_EMBLEM_NAME_SHARED "shared"
 
 typedef void (*NautilusFileCallback)          (NautilusFile  *file,
 				               gpointer       callback_data);
@@ -155,6 +158,7 @@ char *                  nautilus_file_get_display_name                  (Nautilu
 char *                  nautilus_file_get_edit_name                     (NautilusFile                   *file);
 char *                  nautilus_file_get_name                          (NautilusFile                   *file);
 GFile *                 nautilus_file_get_location                      (NautilusFile                   *file);
+char *			 nautilus_file_get_description			 (NautilusFile			 *file);
 char *                  nautilus_file_get_uri                           (NautilusFile                   *file);
 char *                  nautilus_file_get_uri_scheme                    (NautilusFile                   *file);
 NautilusFile *          nautilus_file_get_parent                        (NautilusFile                   *file);
@@ -180,6 +184,7 @@ gboolean                nautilus_file_is_broken_symbolic_link           (Nautilu
 gboolean                nautilus_file_is_nautilus_link                  (NautilusFile                   *file);
 gboolean                nautilus_file_is_executable                     (NautilusFile                   *file);
 gboolean                nautilus_file_is_directory                      (NautilusFile                   *file);
+gboolean		nautilus_file_is_archive			(NautilusFile			*file);
 gboolean                nautilus_file_is_in_trash                       (NautilusFile                   *file);
 gboolean                nautilus_file_is_in_desktop                     (NautilusFile                   *file);
 gboolean		nautilus_file_is_home				(NautilusFile                   *file);
@@ -220,6 +225,10 @@ void                    nautilus_file_set_attributes                    (Nautilu
 									 gpointer                        callback_data);
 GFilesystemPreviewType  nautilus_file_get_filesystem_use_preview        (NautilusFile *file);
 
+char *                  nautilus_file_get_filesystem_id                 (NautilusFile                   *file);
+
+NautilusFile *          nautilus_file_get_trash_original_file           (NautilusFile                   *file);
+
 /* Permissions. */
 gboolean                nautilus_file_can_get_permissions               (NautilusFile                   *file);
 gboolean                nautilus_file_can_set_permissions               (NautilusFile                   *file);
