@@ -260,6 +260,7 @@ void              nautilus_icon_container_select_all                    (Nautilu
 
 /* operations on the selection */
 GList     *       nautilus_icon_container_get_selection                 (NautilusIconContainer  *view);
+void			  nautilus_icon_container_invert_selection				(NautilusIconContainer  *view);
 void              nautilus_icon_container_set_selection                 (NautilusIconContainer  *view,
 									 GList                  *selection);
 GArray    *       nautilus_icon_container_get_selected_icon_locations   (NautilusIconContainer  *view);
@@ -301,8 +302,17 @@ char*             nautilus_icon_container_get_icon_description          (Nautilu
 gboolean          nautilus_icon_container_get_allow_moves               (NautilusIconContainer  *container);
 void              nautilus_icon_container_set_allow_moves               (NautilusIconContainer  *container,
 									 gboolean                allow_moves);
+void		  nautilus_icon_container_set_forced_icon_size		(NautilusIconContainer  *container,
+									 int                     forced_icon_size);
+void		  nautilus_icon_container_set_all_columns_same_width	(NautilusIconContainer  *container,
+									 gboolean                all_columns_same_width);
 
 gboolean	  nautilus_icon_container_is_layout_rtl			(NautilusIconContainer  *container);
+gboolean	  nautilus_icon_container_is_layout_vertical		(NautilusIconContainer  *container);
+
+void              nautilus_icon_container_widget_to_file_operation_position (NautilusIconContainer *container,
+									     GdkPoint              *position);
+
 
 #define CANVAS_WIDTH(container) ((GTK_WIDGET (container)->allocation.width \
 				- container->details->left_margin \
