@@ -21,7 +21,7 @@
  *
  * Ephy port by Marco Pesenti Gritti <marco@it.gnome.org>
  * 
- * $Id: ephy-spinner.c 12639 2006-12-12 17:06:55Z chpe $
+ * $Id: ephy-spinner.c 14328 2008-07-08 21:05:55Z cneumair $
  */
 
 #include "config.h"
@@ -34,9 +34,7 @@
 #define STOP_PROFILER(name)
 
 #include <gdk-pixbuf/gdk-pixbuf.h>
-#include <gtk/gtkicontheme.h>
-#include <gtk/gtkiconfactory.h>
-#include <gtk/gtksettings.h>
+#include <gtk/gtk.h>
 
 /* Spinner cache implementation */
 
@@ -824,6 +822,13 @@ ephy_spinner_set_timeout (EphySpinner *spinner,
 }
 #endif
 
+gboolean
+ephy_spinner_get_spinning (EphySpinner *spinner)
+{
+	EphySpinnerDetails *details = spinner->details;
+	return details->spinning;
+}
+
 static void
 ephy_spinner_size_request (GtkWidget *widget,
 			   GtkRequisition *requisition)
