@@ -30,7 +30,7 @@
 #include "fm-actions.h"
 
 #include <X11/Xatom.h>
-#include <gtk/gtkmain.h>
+#include <gtk/gtk.h>
 #include <dirent.h>
 #include <eel/eel-glib-extensions.h>
 #include <eel/eel-gnome-extensions.h>
@@ -41,10 +41,8 @@
 #include <eel/eel-vfs-extensions.h>
 #include <fcntl.h>
 #include <gdk/gdkx.h>
-#include <gtk/gtkcheckmenuitem.h>
-#include <libgnome/gnome-i18n.h>
+#include <glib/gi18n.h>
 #include <libgnome/gnome-util.h>
-#include <libgnomevfs/gnome-vfs.h>
 #include <libnautilus-private/nautilus-desktop-icon-file.h>
 #include <libnautilus-private/nautilus-directory-background.h>
 #include <libnautilus-private/nautilus-directory-notify.h>
@@ -66,7 +64,6 @@
 #include <sys/stat.h>
 #include <sys/types.h>
 #include <unistd.h>
-#include <gtk/gtkmessagedialog.h>
 
 /* Timeout to check the desktop directory for updates */
 #define RESCAN_TIMEOUT 4000
@@ -181,7 +178,7 @@ net_workarea_changed (FMDesktopIconView *icon_view,
 			       &format_returned,
 			       &length_returned,
 			       (guchar **) &nworkareas)) {
-		g_warning("Can not caclulate _NET_NUMBER_OF_DESKTOPS");
+		g_warning("Can not calculate _NET_NUMBER_OF_DESKTOPS");
 	}
 	if (gdk_error_trap_pop()
 	    || nworkareas == NULL
@@ -271,10 +268,6 @@ fm_desktop_icon_view_destroy (GtkObject *object)
 		icon_view->details->reload_desktop_timeout = 0;
 	}
 
-	eel_preferences_remove_callback (NAUTILUS_PREFERENCES_ICON_VIEW_DEFAULT_ZOOM_LEVEL,
-					 default_zoom_level_changed,
-					 icon_view);
-	
 	ui_manager = fm_directory_view_get_ui_manager (FM_DIRECTORY_VIEW (icon_view));
 	if (ui_manager != NULL) {
 		nautilus_ui_unmerge_ui (ui_manager,
@@ -292,6 +285,10 @@ fm_desktop_icon_view_finalize (GObject *object)
 
 	icon_view = FM_DESKTOP_ICON_VIEW (object);
 	
+	eel_preferences_remove_callback (NAUTILUS_PREFERENCES_ICON_VIEW_DEFAULT_ZOOM_LEVEL,
+					 default_zoom_level_changed,
+					 icon_view);
+	
 	g_free (icon_view->details);
 
 	G_OBJECT_CLASS (parent_class)->finalize (object);
@@ -396,7 +393,7 @@ realized_callback (GtkWidget *widget, FMDesktopIconView *desktop_icon_view)
 	net_workarea_changed (desktop_icon_view, root_window);
 
 	/* Setup the property filter */
-	gdk_window_set_events (root_window, GDK_PROPERTY_NOTIFY);
+	gdk_window_set_events (root_window, GDK_PROPERTY_CHANGE_MASK);
 	gdk_window_add_filter (root_window,
 			       desktop_icon_view_property_filter,
 			       desktop_icon_view);
@@ -568,6 +565,8 @@ fm_desktop_icon_view_init (FMDesktopIconView *desktop_icon_view)
 	
 	/* Set our default layout mode */
 	nautilus_icon_container_set_layout_mode (icon_container,
+						 gtk_widget_get_direction (GTK_WIDGET(icon_container)) == GTK_TEXT_DIR_RTL ?
+						 NAUTILUS_ICON_LAYOUT_T_B_R_L :
 						 NAUTILUS_ICON_LAYOUT_T_B_L_R);
 
 	g_signal_connect_object (icon_container, "middle_click",
@@ -615,8 +614,8 @@ action_change_background_callback (GtkAction *action,
 
 	nautilus_launch_application_from_command (gtk_widget_get_screen (GTK_WIDGET (data)),
 						  _("Background"),
-						  "gnome-background-properties",
-						  NULL,
+						  "gnome-appearance-properties",
+						  "--show-page=background",
 						  FALSE);
 }
 
@@ -685,7 +684,7 @@ real_update_menus (FMDirectoryView *view)
 	}
 }
 
-static GtkActionEntry desktop_view_entries[] = {
+static const GtkActionEntry desktop_view_entries[] = {
 	/* name, stock id */
 	{ "New Launcher Desktop", NULL,
 	  /* label, accelerator */
@@ -770,11 +769,13 @@ real_supports_zooming (FMDirectoryView *view)
 }
 
 static NautilusView *
-fm_desktop_icon_view_create (NautilusWindowInfo *window)
+fm_desktop_icon_view_create (NautilusWindowSlotInfo *slot)
 {
 	FMIconView *view;
 
-	view = g_object_new (FM_TYPE_DESKTOP_ICON_VIEW, "window", window, NULL);
+	view = g_object_new (FM_TYPE_DESKTOP_ICON_VIEW,
+			     "window-slot", slot,
+			     NULL);
 	g_object_ref (view);
 	gtk_object_sink (GTK_OBJECT (view));
 	return NAUTILUS_VIEW (view);
@@ -782,8 +783,8 @@ fm_desktop_icon_view_create (NautilusWindowInfo *window)
 
 static gboolean
 fm_desktop_icon_view_supports_uri (const char *uri,
-			   GnomeVFSFileType file_type,
-			   const char *mime_type)
+				   GFileType file_type,
+				   const char *mime_type)
 {
 	if (g_str_has_prefix (uri, EEL_DESKTOP_URI)) {
 		return TRUE;
@@ -794,12 +795,11 @@ fm_desktop_icon_view_supports_uri (const char *uri,
 
 static NautilusViewInfo fm_desktop_icon_view = {
 	FM_DESKTOP_ICON_VIEW_ID,
-	N_("Desktop"),
-	N_("View as Desktop"),
-	N_("View as _Desktop"),
+	"Desktop View",
+	"_Desktop",
 	N_("The desktop view encountered an error."),
 	N_("The desktop view encountered an error while starting up."),
-	N_("Display this location with the desktop view."),
+	"Display this location with the desktop view.",
 	fm_desktop_icon_view_create,
 	fm_desktop_icon_view_supports_uri
 };
@@ -807,12 +807,8 @@ static NautilusViewInfo fm_desktop_icon_view = {
 void
 fm_desktop_icon_view_register (void)
 {
-	fm_desktop_icon_view.label = _(fm_desktop_icon_view.label);
-	fm_desktop_icon_view.view_as_label = _(fm_desktop_icon_view.view_as_label);
-	fm_desktop_icon_view.view_as_label_with_mnemonic = _(fm_desktop_icon_view.view_as_label_with_mnemonic);
 	fm_desktop_icon_view.error_label = _(fm_desktop_icon_view.error_label);
 	fm_desktop_icon_view.startup_error_label = _(fm_desktop_icon_view.startup_error_label);
-	fm_desktop_icon_view.display_location_label = _(fm_desktop_icon_view.display_location_label);
 	
 	nautilus_view_factory_register (&fm_desktop_icon_view);
 }
