@@ -40,25 +40,12 @@
 #include <eel/eel-gtk-extensions.h>
 #include <eel/eel-preferences.h>
 #include <eel/eel-stock-dialogs.h>
-#include <gtk/gtkmain.h>
-#include <gtk/gtkcellrendererpixbuf.h>
-#include <gtk/gtkcellrenderertext.h>
-#include <gtk/gtkscrolledwindow.h>
-#include <gtk/gtktreemodelsort.h>
-#include <gtk/gtktreeselection.h>
-#include <gtk/gtktreeview.h>
-#include <gtk/gtkstock.h>
-#include <gtk/gtkimage.h>
-#include <gtk/gtkimagemenuitem.h>
-#include <gtk/gtkseparatormenuitem.h>
-#include <gtk/gtklabel.h>
-#include <gtk/gtkmenu.h>
-#include <gtk/gtkmenushell.h>
-#include <gtk/gtkclipboard.h>
+#include <gtk/gtk.h>
 #include <glib/gi18n.h>
 #include <gio/gio.h>
 #include <libgnomeui/gnome-uidefs.h>
 #include <libgnomeui/gnome-popup-menu.h>
+#include <libnautilus-private/nautilus-clipboard.h>
 #include <libnautilus-private/nautilus-clipboard-monitor.h>
 #include <libnautilus-private/nautilus-desktop-icon-file.h>
 #include <libnautilus-private/nautilus-debug-log.h>
@@ -72,6 +59,8 @@
 #include <libnautilus-private/nautilus-cell-renderer-pixbuf-emblem.h>
 #include <libnautilus-private/nautilus-sidebar-provider.h>
 #include <libnautilus-private/nautilus-module.h>
+#include <libnautilus-private/nautilus-window-info.h>
+#include <libnautilus-private/nautilus-window-slot-info.h>
 
 typedef struct {
         GObject parent;
@@ -91,7 +80,7 @@ struct FMTreeViewDetails {
 	GVolumeMonitor *volume_monitor;
 
 	NautilusFile *activation_file;
-	gboolean activation_in_new_window;
+	NautilusWindowOpenFlags activation_flags;
 
 	NautilusTreeViewDragDest *drag_dest;
 
@@ -126,16 +115,11 @@ typedef struct {
 static GdkAtom copied_files_atom;
 static gboolean show_delete_command_auto_value;
 
-enum {
-	GNOME_COPIED_FILES
-};
-
-static const GtkTargetEntry clipboard_targets[] = {
-	{ "x-special/gnome-copied-files", 0, GNOME_COPIED_FILES },
-};
-
 static void  fm_tree_view_iface_init        (NautilusSidebarIface         *iface);
 static void  sidebar_provider_iface_init    (NautilusSidebarProviderIface *iface);
+static void  fm_tree_view_activate_file     (FMTreeView *view, 
+			    		     NautilusFile *file,
+					     NautilusWindowOpenFlags flags);
 static GType fm_tree_view_provider_get_type (void);
 
 static void create_popup_menu (FMTreeView *view);
@@ -335,16 +319,22 @@ got_activation_uri_callback (NautilusFile *file, gpointer callback_data)
         char *uri, *file_uri;
         FMTreeView *view;
 	GdkScreen *screen;
-	NautilusWindowOpenMode mode;
 	GFile *location;
+	NautilusWindowSlotInfo *slot;
+	gboolean open_in_same_slot;
 	
         view = FM_TREE_VIEW (callback_data);
 
 	screen = gtk_widget_get_screen (GTK_WIDGET (view->details->tree_widget));
 
         g_assert (file == view->details->activation_file);
-        
-        mode = view->details->activation_in_new_window ? NAUTILUS_WINDOW_OPEN_IN_NAVIGATION : NAUTILUS_WINDOW_OPEN_ACCORDING_TO_MODE;
+
+	open_in_same_slot =
+		(view->details->activation_flags &
+		 (NAUTILUS_WINDOW_OPEN_FLAG_NEW_WINDOW |
+		  NAUTILUS_WINDOW_OPEN_FLAG_NEW_TAB)) == 0;
+
+	slot = nautilus_window_info_get_active_slot (view->details->window);
 
 	uri = nautilus_file_get_activation_uri (file);
 	if (nautilus_file_is_launcher (file)) {
@@ -367,11 +357,11 @@ got_activation_uri_callback (NautilusFile *file, gpointer callback_data)
 					    "tree view window_info_open_location window=%p: %s",
 					    view->details->window, uri);
 			location = g_file_new_for_uri (uri);
-			nautilus_window_info_open_location
-				(view->details->window, 
+			nautilus_window_slot_info_open_location
+				(slot,
 				 location, 
-				 mode,
-				 0,
+				 NAUTILUS_WINDOW_OPEN_ACCORDING_TO_MODE,
+				 view->details->activation_flags,
 				 NULL);
 			g_object_unref (location);
 		} else {
@@ -383,21 +373,25 @@ got_activation_uri_callback (NautilusFile *file, gpointer callback_data)
 		}
 		   
 	} else if (uri != NULL) {
-		if (view->details->selection_location == NULL ||
+		if (!open_in_same_slot ||
+		    view->details->selection_location == NULL ||
 		    strcmp (uri, view->details->selection_location) != 0) {
-			if (view->details->selection_location != NULL) {
-				g_free (view->details->selection_location);
+			if (open_in_same_slot) {
+				if (view->details->selection_location != NULL) {
+					g_free (view->details->selection_location);
+				}
+				view->details->selection_location = g_strdup (uri);
 			}
-			view->details->selection_location = g_strdup (uri);
+
 			nautilus_debug_log (FALSE, NAUTILUS_DEBUG_LOG_DOMAIN_USER,
 					    "tree view window_info_open_location window=%p: %s",
 					    view->details->window, uri);
 			location = g_file_new_for_uri (uri);
-			nautilus_window_info_open_location
-				(view->details->window, 
+			nautilus_window_slot_info_open_location
+				(slot,
 				 location,
-				 mode,
-				 0,
+				 NAUTILUS_WINDOW_OPEN_ACCORDING_TO_MODE,
+				 view->details->activation_flags,
 				 NULL);
 			g_object_unref (location);
 		}
@@ -458,7 +452,7 @@ selection_changed_timer_callback(FMTreeView *view)
 	if (view->details->activation_file == NULL) {
 		return FALSE;
 	}
-	view->details->activation_in_new_window = FALSE;
+	view->details->activation_flags = 0;
 		
 	attributes = NAUTILUS_FILE_ATTRIBUTE_INFO | NAUTILUS_FILE_ATTRIBUTE_LINK_INFO;
 	nautilus_file_call_when_ready (view->details->activation_file, attributes,
@@ -572,7 +566,10 @@ move_copy_items_callback (NautilusTreeViewDragDest *dest,
 	FMTreeView *view;
 
 	view = FM_TREE_VIEW (user_data);
-
+	
+	nautilus_clipboard_clear_if_colliding_uris (GTK_WIDGET (view),
+						    item_uris,
+						    copied_files_atom);
 	nautilus_file_operations_copy_move
 		(item_uris,
 		 NULL,
@@ -647,13 +644,6 @@ clipboard_contents_received_callback (GtkClipboard     *clipboard,
 	g_object_unref (view);
 }
 
-static GtkClipboard *
-get_clipboard (GtkWidget *widget)
-{
-	return gtk_clipboard_get_for_display (gtk_widget_get_display (widget),
-					      GDK_SELECTION_CLIPBOARD);
-}
-
 static gboolean
 is_parent_writable (NautilusFile *file)
 {
@@ -713,7 +703,7 @@ button_pressed_callback (GtkTreeView *treeview, GdkEventButton *event,
 		gtk_widget_set_sensitive (view->details->popup_paste, FALSE);
 		if (nautilus_file_is_directory (view->details->popup_file) &&
 			nautilus_file_can_write (view->details->popup_file)) {
-			gtk_clipboard_request_contents (get_clipboard (GTK_WIDGET (view->details->tree_widget)),
+			gtk_clipboard_request_contents (nautilus_clipboard_get (GTK_WIDGET (view->details->tree_widget)),
 							copied_files_atom,
 							clipboard_contents_received_callback, g_object_ref (view));
 		}
@@ -769,6 +759,26 @@ button_pressed_callback (GtkTreeView *treeview, GdkEventButton *event,
 		
 		g_object_unref (view);
 
+		return TRUE;
+	} else if (event->button == 2 && event->type == GDK_BUTTON_PRESS) {
+		NautilusFile *file;
+
+		if (!gtk_tree_view_get_path_at_pos (treeview, event->x, event->y,
+						    &path, NULL, NULL, NULL)) {
+			return FALSE;
+		}
+
+		file = sort_model_path_to_file (view, path);
+		if (file) {
+			fm_tree_view_activate_file (view, file, 
+						    (event->state & GDK_CONTROL_MASK) != 0 ?
+						    NAUTILUS_WINDOW_OPEN_FLAG_NEW_WINDOW :
+						    NAUTILUS_WINDOW_OPEN_FLAG_NEW_TAB);
+			nautilus_file_unref (file);
+		}
+
+		gtk_tree_path_free (path);
+
 		return TRUE;
 	}
 
@@ -778,14 +788,21 @@ button_pressed_callback (GtkTreeView *treeview, GdkEventButton *event,
 static void
 fm_tree_view_activate_file (FMTreeView *view, 
 			    NautilusFile *file,
-			    gboolean open_in_new_window)
+			    NautilusWindowOpenFlags flags)
 {
 	NautilusFileAttributes attributes;
 
 	cancel_activation (view);
 
+	if (view->details->activation_flags & NAUTILUS_WINDOW_OPEN_FLAG_NEW_TAB &&
+	    !eel_preferences_get_boolean (NAUTILUS_PREFERENCES_ENABLE_TABS)) {
+		view->details->activation_flags &= ~NAUTILUS_WINDOW_OPEN_FLAG_NEW_TAB;
+		view->details->activation_flags |= NAUTILUS_WINDOW_OPEN_FLAG_NEW_WINDOW;
+	}
+
+
 	view->details->activation_file = nautilus_file_ref (file);
-	view->details->activation_in_new_window = open_in_new_window;
+	view->details->activation_flags = flags;
 		
 	attributes = NAUTILUS_FILE_ATTRIBUTE_INFO | NAUTILUS_FILE_ATTRIBUTE_LINK_INFO;
 	nautilus_file_call_when_ready (view->details->activation_file, attributes,
@@ -796,14 +813,21 @@ static void
 fm_tree_view_open_cb (GtkWidget *menu_item,
 		      FMTreeView *view)
 {
-	fm_tree_view_activate_file (view, view->details->popup_file, FALSE);
+	fm_tree_view_activate_file (view, view->details->popup_file, 0);
+}
+
+static void
+fm_tree_view_open_in_new_tab_cb (GtkWidget *menu_item,
+				    FMTreeView *view)
+{
+	fm_tree_view_activate_file (view, view->details->popup_file, NAUTILUS_WINDOW_OPEN_FLAG_NEW_TAB);
 }
 
 static void
 fm_tree_view_open_in_new_window_cb (GtkWidget *menu_item,
 				    FMTreeView *view)
 {
-	fm_tree_view_activate_file (view, view->details->popup_file, TRUE);
+	fm_tree_view_activate_file (view, view->details->popup_file, NAUTILUS_WINDOW_OPEN_FLAG_NEW_WINDOW);
 }
 
 static void
@@ -836,46 +860,74 @@ fm_tree_view_create_folder_cb (GtkWidget *menu_item,
 	g_free (parent_uri);
 }
 
+typedef struct {
+        char *file_uri;
+	gboolean cut;
+} ClipboardInfo;
+
 static void
 get_clipboard_callback (GtkClipboard	 *clipboard,
 			GtkSelectionData *selection_data,
-			guint		  info,
-			gpointer	  user_data_or_owner)
+			guint		  info_,
+			gpointer	  user_data)
 {
-	char *str = user_data_or_owner;
-	
-	gtk_selection_data_set (selection_data,
-				copied_files_atom,
-				8,
-				str,
-				strlen (str));
+	ClipboardInfo *info = user_data;
+
+
+        if (gtk_targets_include_uri (&selection_data->target, 1)) {
+                char *uris[2];
+
+                uris[0] = info->file_uri;
+                uris[1] = NULL;
+                gtk_selection_data_set_uris (selection_data, uris);
+        } else if (gtk_targets_include_text (&selection_data->target, 1)) {
+                GFile *file;
+                char *str;
+
+                file = g_file_new_for_uri (info->file_uri);
+                str = g_file_get_parse_name (file);
+                g_object_unref (file);
+
+                if (str) {
+                        gtk_selection_data_set_text (selection_data, str, strlen (str));
+                        g_free (str);
+                } else {
+                        gtk_selection_data_set_text (selection_data,
+                                                    info->file_uri, strlen (info->file_uri));
+                }
+        } else if (selection_data->target == copied_files_atom) {
+                char *str;
+
+                str = g_strdup_printf ("%s\n%s",
+                                       info->cut ? "cut" : "copy",
+                                       info->file_uri);
+                gtk_selection_data_set (selection_data, copied_files_atom,
+                                        8, str, strlen (str));
+                g_free (str);
+        }
 }
 
 static void
 clear_clipboard_callback (GtkClipboard *clipboard,
-			  gpointer	user_data_or_owner)
+			  gpointer	user_data)
 {
-	g_free (user_data_or_owner);
+	ClipboardInfo *info = user_data;
+
+        g_free (info->file_uri);
+	g_slice_free (ClipboardInfo, info);
 }
 
-static char *
-convert_file_to_string (NautilusFile *file,
-			gboolean cut)
+static ClipboardInfo *
+convert_file_to_uri (NautilusFile *file,
+		     gboolean cut)
 {
-	GString *uris;
-	char *uri, *result;
+        ClipboardInfo *info;
 
-	uris = g_string_new (cut ? "cut" : "copy");
-	
-	uri = nautilus_file_get_uri (file);
-	g_string_append_c (uris, '\n');
-	g_string_append (uris, uri);
-	g_free (uri);
+        info = g_slice_new (ClipboardInfo);
+        info->cut = cut;
+	info->file_uri = nautilus_file_get_uri (file);
 
-	result = uris->str;
-	g_string_free (uris, FALSE);
-
-	return result;
+        return info;
 }
 
 static void
@@ -883,14 +935,27 @@ copy_or_cut_files (FMTreeView *view,
 		   gboolean cut)
 {
 	char *status_string, *name;
-	char *clipboard_string;
+        ClipboardInfo *info;
+        GtkTargetList *target_list;
+        GtkTargetEntry *targets;
+        int n_targets;
+
+	info = convert_file_to_uri (view->details->popup_file, cut);
 	
-	clipboard_string = convert_file_to_string (view->details->popup_file, cut);
-	
-	gtk_clipboard_set_with_data (get_clipboard (GTK_WIDGET (view->details->tree_widget)),
-				     clipboard_targets, G_N_ELEMENTS (clipboard_targets),
+        target_list = gtk_target_list_new (NULL, 0);
+        gtk_target_list_add (target_list, copied_files_atom, 0, 0);
+        gtk_target_list_add_uri_targets (target_list, 0);
+        gtk_target_list_add_text_targets (target_list, 0);
+
+        targets = gtk_target_table_new_from_list (target_list, &n_targets);
+        gtk_target_list_unref (target_list);
+
+	gtk_clipboard_set_with_data (nautilus_clipboard_get (GTK_WIDGET (view->details->tree_widget)),
+				     targets, n_targets,
 				     get_clipboard_callback, clear_clipboard_callback,
-				     clipboard_string);
+				     info);
+        gtk_target_table_free (targets, n_targets);
+
 	nautilus_clipboard_monitor_emit_changed ();
 	
 	name = nautilus_file_get_display_name (view->details->popup_file);
@@ -905,8 +970,8 @@ copy_or_cut_files (FMTreeView *view,
 	}
 	g_free (name);
 	
-	nautilus_window_info_set_status (view->details->window,
-					 status_string);
+	nautilus_window_info_push_status (view->details->window,
+					  status_string);
 	g_free (status_string);
 }
 
@@ -924,65 +989,34 @@ fm_tree_view_copy_cb (GtkWidget *menu_item,
 	copy_or_cut_files (view, FALSE);
 }
 
-static GList *
-convert_lines_to_str_list (char **lines, gboolean *cut)
-{
-	int i;
-	GList *result;
-
-	*cut = FALSE;
-
-	if (lines[0] == NULL) {
-		return NULL;
-	}
-
-	if (strcmp (lines[0], "cut") == 0) {
-		*cut = TRUE;
-	} else if (strcmp (lines[0], "copy") != 0) {
-		return NULL;
-	}
-
-	result = NULL;
-	for (i = 1; lines[i] != NULL; i++) {
-		result = g_list_prepend (result, g_strdup (lines[i]));
-	}
-	return g_list_reverse (result);
-}
-
 static void
 paste_clipboard_data (FMTreeView *view,
 		      GtkSelectionData *selection_data,
 		      char *destination_uri)
 {
-	char **lines;
 	gboolean cut;
 	GList *item_uris;
 
 	cut = FALSE;
-	if (selection_data->type != copied_files_atom
-	    || selection_data->length <= 0) {
-		item_uris = NULL;
-	} else {
-		/* Not sure why it's legal to assume there's an extra byte
-		 * past the end of the selection data that it's safe to write
-		 * to. But gtk_editable_selection_received does this, so I
-		 * think it is OK.
-		 */
-		selection_data->data[selection_data->length] = '\0';
-		lines = g_strsplit (selection_data->data, "\n", 0);
-		item_uris = convert_lines_to_str_list (lines, &cut);
-		g_strfreev (lines);
-	}
+	item_uris = nautilus_clipboard_get_uri_list_from_selection_data (selection_data, &cut,
+									 copied_files_atom);
 
 	if (item_uris == NULL|| destination_uri == NULL) {
-		nautilus_window_info_set_status (view->details->window,
-						 _("There is nothing on the clipboard to paste."));
+		nautilus_window_info_push_status (view->details->window,
+						  _("There is nothing on the clipboard to paste."));
 	} else {
 		nautilus_file_operations_copy_move
 			(item_uris, NULL, destination_uri,
 			 cut ? GDK_ACTION_MOVE : GDK_ACTION_COPY,
 			 GTK_WIDGET (view->details->tree_widget),
 			 NULL, NULL);
+
+		/* If items are cut then remove from clipboard */
+		if (cut) {
+			gtk_clipboard_clear (nautilus_clipboard_get (GTK_WIDGET (view)));
+		}
+		
+		eel_g_list_free_deep (item_uris);
 	}
 }
 
@@ -1007,7 +1041,7 @@ static void
 fm_tree_view_paste_cb (GtkWidget *menu_item,
 		       FMTreeView *view)
 {
-	gtk_clipboard_request_contents (get_clipboard (GTK_WIDGET (view->details->tree_widget)),
+	gtk_clipboard_request_contents (nautilus_clipboard_get (GTK_WIDGET (view->details->tree_widget)),
 					copied_files_atom,
 					paste_into_clipboard_received_callback, view);
 }
@@ -1122,8 +1156,17 @@ create_popup_menu (FMTreeView *view)
 	gtk_menu_shell_append (GTK_MENU_SHELL (popup), menu_item);
 	view->details->popup_open = menu_item;
 	
+	/* add the "open in new tab" menu item */
+	menu_item = gtk_menu_item_new_with_mnemonic (_("Open in New _Tab"));
+	g_signal_connect (menu_item, "activate",
+			  G_CALLBACK (fm_tree_view_open_in_new_tab_cb),
+			  view);
+	gtk_widget_show (menu_item);
+	gtk_menu_shell_append (GTK_MENU_SHELL (popup), menu_item);
+	view->details->popup_open_in_new_window = menu_item;
+	
 	/* add the "open in new window" menu item */
-	menu_item = gtk_image_menu_item_new_with_label (_("Open in New Window"));
+	menu_item = gtk_menu_item_new_with_mnemonic (_("Open in New _Window"));
 	g_signal_connect (menu_item, "activate",
 			  G_CALLBACK (fm_tree_view_open_in_new_window_cb),
 			  view);
@@ -1238,6 +1281,7 @@ create_tree (FMTreeView *view)
 	GList *mounts, *l;
 	char *location;
 	GIcon *icon;
+	NautilusWindowSlotInfo *slot;
 	
 	view->details->child_model = fm_tree_model_new ();
 	view->details->sort_model = GTK_TREE_MODEL_SORT
@@ -1338,7 +1382,8 @@ create_tree (FMTreeView *view)
 			  "button_press_event", G_CALLBACK (button_pressed_callback),
 			  view);
 
-	location = nautilus_window_info_get_current_location (view->details->window);
+	slot = nautilus_window_info_get_active_slot (view->details->window);
+	location = nautilus_window_slot_info_get_current_location (slot);
 	schedule_select_and_show_location (view, location);
 	g_free (location);
 }
@@ -1363,9 +1408,6 @@ update_filtering_from_preferences (FMTreeView *view)
 			(view->details->child_model,
 			 mode == NAUTILUS_WINDOW_SHOW_HIDDEN_FILES_ENABLE);
 	}
-	fm_tree_model_set_show_backup_files
-		(view->details->child_model,
-		 eel_preferences_get_boolean (NAUTILUS_PREFERENCES_SHOW_BACKUP_FILES));
 	fm_tree_model_set_show_only_directories
 		(view->details->child_model,
 		 eel_preferences_get_boolean (NAUTILUS_PREFERENCES_TREE_SHOW_ONLY_DIRECTORIES));
@@ -1395,8 +1437,11 @@ filtering_changed_callback (gpointer callback_data)
 static void
 loading_uri_callback (NautilusWindowInfo *window,
 		      char *location,
-		      FMTreeView *view)
+		      gpointer callback_data)
 {
+	FMTreeView *view;
+
+	view = FM_TREE_VIEW (callback_data);
 	schedule_select_and_show_location (view, location);
 }
 
@@ -1556,12 +1601,15 @@ fm_tree_view_set_parent_window (FMTreeView *sidebar,
 				NautilusWindowInfo *window)
 {
 	char *location;
+	NautilusWindowSlotInfo *slot;
 	
 	sidebar->details->window = window;
 
+	slot = nautilus_window_info_get_active_slot (window);
+
 	g_signal_connect_object (window, "loading_uri",
 				 G_CALLBACK (loading_uri_callback), sidebar, 0);
-	location = nautilus_window_info_get_current_location (window);
+	location = nautilus_window_slot_info_get_current_location (slot);
 	loading_uri_callback (window, location, sidebar);
 	g_free (location);
 
