@@ -30,18 +30,7 @@
 #include <eel/eel-string.h>
 #include <eel/eel-stock-dialogs.h>
 #include <gdk/gdkkeysyms.h>
-#include <gtk/gtkalignment.h>
-#include <gtk/gtkbutton.h>
-#include <gtk/gtkvbox.h>
-#include <gtk/gtkcellrendererpixbuf.h>
-#include <gtk/gtkcellrenderertext.h>
-#include <gtk/gtkliststore.h>
-#include <gtk/gtkmain.h>
-#include <gtk/gtksizegroup.h>
-#include <gtk/gtkstock.h>
-#include <gtk/gtktreemodel.h>
-#include <gtk/gtktreeselection.h>
-#include <gtk/gtkimagemenuitem.h>
+#include <gtk/gtk.h>
 #include <libgnome/gnome-macros.h>
 #include <glib/gi18n.h>
 #include <libgnomeui/gnome-popup-menu.h>
@@ -57,12 +46,16 @@
 #include <libnautilus-private/nautilus-trash-monitor.h>
 #include <libnautilus-private/nautilus-icon-names.h>
 #include <libnautilus-private/nautilus-autorun.h>
+#include <libnautilus-private/nautilus-window-info.h>
+#include <libnautilus-private/nautilus-window-slot-info.h>
 #include <gio/gio.h>
 
 #include "nautilus-bookmark-list.h"
 #include "nautilus-places-sidebar.h"
 #include "nautilus-window.h"
 
+#define EJECT_BUTTON_XPAD 5
+
 #define NAUTILUS_PLACES_SIDEBAR_CLASS(klass)    (GTK_CHECK_CLASS_CAST ((klass), NAUTILUS_TYPE_PLACES_SIDEBAR, NautilusPlacesSidebarClass))
 #define NAUTILUS_IS_PLACES_SIDEBAR(obj)         (GTK_CHECK_TYPE ((obj), NAUTILUS_TYPE_PLACES_SIDEBAR))
 #define NAUTILUS_IS_PLACES_SIDEBAR_CLASS(klass) (GTK_CHECK_CLASS_TYPE ((klass), NAUTILUS_TYPE_PLACES_SIDEBAR))
@@ -70,6 +63,8 @@
 typedef struct {
 	GtkScrolledWindow  parent;
 	GtkTreeView        *tree_view;
+	GtkCellRenderer    *icon_cell_renderer;
+	GtkCellRenderer    *eject_text_cell_renderer;
 	char 	           *uri;
 	GtkListStore       *store;
 	GtkTreeModel       *filter_model;
@@ -80,9 +75,11 @@ typedef struct {
 	/* DnD */
 	GList     *drag_list;
 	gboolean  drag_data_received;
+	int       drag_data_info;
 	gboolean  drop_occured;
 
 	GtkWidget *popup_menu;
+	GtkWidget *popup_menu_open_in_new_tab_item;
 	GtkWidget *popup_menu_remove_item;
 	GtkWidget *popup_menu_rename_item;
 	GtkWidget *popup_menu_separator_item;
@@ -92,6 +89,11 @@ typedef struct {
 	GtkWidget *popup_menu_rescan_item;
 	GtkWidget *popup_menu_format_item;
 	GtkWidget *popup_menu_empty_trash_item;
+
+	/* volume mounting - delayed open process */
+	gboolean mounting;
+	NautilusWindowSlotInfo *go_to_after_mount_slot;
+	NautilusWindowOpenFlags go_to_after_mount_flags;
 } NautilusPlacesSidebar;
 
 typedef struct {
@@ -115,6 +117,10 @@ enum {
 	PLACES_SIDEBAR_COLUMN_NAME,
 	PLACES_SIDEBAR_COLUMN_ICON,
 	PLACES_SIDEBAR_COLUMN_INDEX,
+	PLACES_SIDEBAR_COLUMN_EJECT,
+	PLACES_SIDEBAR_COLUMN_NO_EJECT,
+	PLACES_SIDEBAR_COLUMN_BOOKMARK,
+	PLACES_SIDEBAR_COLUMN_NO_BOOKMARK,
 	
 	PLACES_SIDEBAR_COLUMN_COUNT
 };
@@ -132,9 +138,19 @@ static GType nautilus_places_sidebar_provider_get_type (void);
 static void  open_selected_bookmark                    (NautilusPlacesSidebar        *sidebar,
 							GtkTreeModel                 *model,
 							GtkTreePath                  *path,
-							gboolean                      open_in_new_window);
+							NautilusWindowOpenFlags flags);
 static void  nautilus_places_sidebar_style_set         (GtkWidget                    *widget,
 							GtkStyle                     *previous_style);
+static gboolean eject_or_unmount_bookmark              (NautilusPlacesSidebar *sidebar,
+							GtkTreePath *path);
+static gboolean eject_or_unmount_selection             (NautilusPlacesSidebar *sidebar);
+static void  check_unmount_and_eject                   (GMount *mount,
+							GVolume *volume,
+							GDrive *drive,
+							gboolean *show_unmount,
+							gboolean *show_eject);
+
+static void bookmarks_check_popup_sensitivity          (NautilusPlacesSidebar *sidebar);
 
 /* Identifiers for target types */
 enum {
@@ -203,12 +219,20 @@ add_place (NautilusPlacesSidebar *sidebar,
 	GtkTreeIter           iter, child_iter;
 	NautilusIconInfo *icon_info;
 	int icon_size;
+	gboolean show_eject, show_unmount;
 
 	icon_size = nautilus_get_icon_size_for_stock_size (GTK_ICON_SIZE_MENU);
 	icon_info = nautilus_icon_info_lookup (icon, icon_size);
 
 	pixbuf = nautilus_icon_info_get_pixbuf_at_size (icon_info, icon_size);
 	g_object_unref (icon_info);
+	check_unmount_and_eject (mount, volume, drive,
+				 &show_unmount, &show_eject);
+
+	if (show_unmount || show_eject) {
+		g_assert (place_type != PLACES_BOOKMARK);
+	}
+ 		  
 	gtk_list_store_append (sidebar->store, &iter);
 	gtk_list_store_set (sidebar->store, &iter,
 			    PLACES_SIDEBAR_COLUMN_ICON, pixbuf,
@@ -219,7 +243,12 @@ add_place (NautilusPlacesSidebar *sidebar,
 			    PLACES_SIDEBAR_COLUMN_MOUNT, mount,
 			    PLACES_SIDEBAR_COLUMN_ROW_TYPE, place_type,
 			    PLACES_SIDEBAR_COLUMN_INDEX, index,
+			    PLACES_SIDEBAR_COLUMN_EJECT, (show_unmount || show_eject),
+			    PLACES_SIDEBAR_COLUMN_NO_EJECT, !(show_unmount || show_eject),
+			    PLACES_SIDEBAR_COLUMN_BOOKMARK, place_type != PLACES_BOOKMARK,
+			    PLACES_SIDEBAR_COLUMN_NO_BOOKMARK, place_type == PLACES_BOOKMARK,
 			    -1);
+
 	if (pixbuf != NULL) {
 		g_object_unref (pixbuf);
 	}
@@ -247,11 +276,13 @@ update_places (NautilusPlacesSidebar *sidebar)
 	char *location, *mount_uri, *name, *desktop_path;
 	GIcon *icon;
 	GFile *root;
-	
-		
+	NautilusWindowSlotInfo *slot;
+
 	selection = gtk_tree_view_get_selection (sidebar->tree_view);
 	gtk_list_store_clear (sidebar->store);
-	location = nautilus_window_info_get_current_location (sidebar->window);
+
+	slot = nautilus_window_info_get_active_slot (sidebar->window);
+	location = nautilus_window_slot_info_get_current_location (slot);
 
 	/* add built in bookmarks */
 	desktop_path = nautilus_get_desktop_directory ();
@@ -267,7 +298,7 @@ update_places (NautilusPlacesSidebar *sidebar)
 				       mount_uri, NULL, NULL, NULL, 0);
 		g_object_unref (icon);
 		g_free (display_name);
-		if (strcmp (location, mount_uri) == 0) {
+		if (eel_strcmp (location, mount_uri) == 0) {
 			gtk_tree_selection_select_iter (selection, &last_iter);
 		}	
 		g_free (mount_uri);
@@ -279,7 +310,7 @@ update_places (NautilusPlacesSidebar *sidebar)
 			       _("Desktop"), icon,
 			       mount_uri, NULL, NULL, NULL, 0);
 	g_object_unref (icon);
-	if (strcmp (location, mount_uri) == 0) {
+	if (eel_strcmp (location, mount_uri) == 0) {
 		gtk_tree_selection_select_iter (selection, &last_iter);
 	}	
 	g_free (mount_uri);
@@ -291,6 +322,16 @@ update_places (NautilusPlacesSidebar *sidebar)
 			       _("File System"), icon,
 			       mount_uri, NULL, NULL, NULL, 0);
 	g_object_unref (icon);
+	if (eel_strcmp (location, mount_uri) == 0) {
+		gtk_tree_selection_select_iter (selection, &last_iter);
+	}
+
+ 	mount_uri = "network:///"; /* No need to strdup */
+	icon = g_themed_icon_new (NAUTILUS_ICON_NETWORK);
+	last_iter = add_place (sidebar, PLACES_BUILT_IN,
+			       _("Network"), icon,
+			       mount_uri, NULL, NULL, NULL, 0);
+	g_object_unref (icon);
 	if (strcmp (location, mount_uri) == 0) {
 		gtk_tree_selection_select_iter (selection, &last_iter);
 	}
@@ -317,7 +358,7 @@ update_places (NautilusPlacesSidebar *sidebar)
 					last_iter = add_place (sidebar, PLACES_MOUNTED_VOLUME,
 							       name, icon, mount_uri,
 							       drive, volume, mount, 0);
-					if (strcmp (location, mount_uri) == 0) {
+					if (eel_strcmp (location, mount_uri) == 0) {
 						gtk_tree_selection_select_iter (selection, &last_iter);
 					}
 					g_object_unref (mount);
@@ -387,7 +428,7 @@ update_places (NautilusPlacesSidebar *sidebar)
 			last_iter = add_place (sidebar, PLACES_MOUNTED_VOLUME,
 					       name, icon, mount_uri,
 					       NULL, volume, mount, 0);
-			if (strcmp (location, mount_uri) == 0) {
+			if (eel_strcmp (location, mount_uri) == 0) {
 				gtk_tree_selection_select_iter (selection, &last_iter);
 			}
 			g_object_unref (mount);
@@ -426,7 +467,7 @@ update_places (NautilusPlacesSidebar *sidebar)
 		last_iter = add_place (sidebar, PLACES_MOUNTED_VOLUME,
 				       name, icon, mount_uri,
 				       NULL, NULL, mount, 0);
-		if (strcmp (location, mount_uri) == 0) {
+		if (eel_strcmp (location, mount_uri) == 0) {
 			gtk_tree_selection_select_iter (selection, &last_iter);
 		}
 		g_object_unref (mount);
@@ -441,7 +482,7 @@ update_places (NautilusPlacesSidebar *sidebar)
 	last_iter = add_place (sidebar, PLACES_BUILT_IN,
 			       _("Trash"), icon, mount_uri,
 			       NULL, NULL, NULL, 0);
-	if (strcmp (location, mount_uri) == 0) {
+	if (eel_strcmp (location, mount_uri) == 0) {
 		gtk_tree_selection_select_iter (selection, &last_iter);
 	}
 	g_object_unref (icon);
@@ -469,7 +510,7 @@ update_places (NautilusPlacesSidebar *sidebar)
 		last_iter = add_place (sidebar, PLACES_BOOKMARK,
 				       name, icon, mount_uri,
 				       NULL, NULL, NULL, index);
-		if (strcmp (location, mount_uri) == 0) {
+		if (eel_strcmp (location, mount_uri) == 0) {
 			gtk_tree_selection_select_iter (selection, &last_iter);
 		}
 		g_free (name);
@@ -567,6 +608,62 @@ drive_changed_callback (GVolumeMonitor *volume_monitor,
 	update_places (sidebar);
 }
 
+static gboolean
+clicked_eject_button (NautilusPlacesSidebar *sidebar,
+		      GtkTreePath **path)
+{
+	GdkEvent *event = gtk_get_current_event ();
+	GdkEventButton *button_event = (GdkEventButton *) event;
+	GtkTreeViewColumn *column;
+	GtkTextDirection direction;
+	int width, total_width;
+	int eject_button_size;
+
+	*path = NULL;
+
+	if (event->type == GDK_BUTTON_PRESS &&
+	    gtk_tree_view_get_path_at_pos (sidebar->tree_view,
+					   button_event->x, button_event->y,
+					   path, &column, NULL, NULL)) {
+		total_width = 0;
+
+		gtk_widget_style_get (GTK_WIDGET (sidebar->tree_view),
+				      "horizontal-separator", &width,
+				      NULL);
+		total_width += width / 2;
+
+		direction = gtk_widget_get_direction (GTK_WIDGET (sidebar->tree_view));
+		if (direction != GTK_TEXT_DIR_RTL) {
+			gtk_tree_view_column_cell_get_position (column,
+								sidebar->icon_cell_renderer,
+								NULL, &width);
+			total_width += width;
+
+			gtk_tree_view_column_cell_get_position (column,
+								sidebar->eject_text_cell_renderer,
+								NULL, &width);
+			total_width += width;
+		}
+
+		total_width += EJECT_BUTTON_XPAD;
+
+		eject_button_size = nautilus_get_icon_size_for_stock_size (GTK_ICON_SIZE_MENU);
+
+		if (button_event->x - total_width >= 0 &&
+		    button_event->x - total_width <= eject_button_size) {
+			return TRUE;
+		}
+	} else if (event->type == GDK_BUTTON_RELEASE) {
+		g_object_set_data (G_OBJECT (sidebar->tree_view), "myrect", NULL);
+	}
+
+	if (*path != NULL) {
+		gtk_tree_path_free (*path);
+	}
+
+	return FALSE;
+}
+
 static void
 row_activated_callback (GtkTreeView *tree_view,
 			GtkTreePath *path,
@@ -576,28 +673,7 @@ row_activated_callback (GtkTreeView *tree_view,
 	open_selected_bookmark (NAUTILUS_PLACES_SIDEBAR (user_data),
 				gtk_tree_view_get_model (tree_view),
 				path,
-				FALSE);
-}
-
-static void
-update_click_policy (NautilusPlacesSidebar *sidebar)
-{
-	int policy;
-	
-	policy = eel_preferences_get_enum (NAUTILUS_PREFERENCES_CLICK_POLICY);
-	
-	eel_gtk_tree_view_set_activate_on_single_click
-		(sidebar->tree_view, policy == NAUTILUS_CLICK_POLICY_SINGLE);
-}
-
-static void
-click_policy_changed_callback (gpointer user_data)
-{
-	NautilusPlacesSidebar *sidebar;
-	
-	sidebar = NAUTILUS_PLACES_SIDEBAR (user_data);
-
-	update_click_policy (sidebar);
+				0);
 }
 
 static void
@@ -610,6 +686,16 @@ desktop_location_changed_callback (gpointer user_data)
 	update_places (sidebar);
 }
 
+static void
+enable_tabs_changed_callback (gpointer user_data)
+{
+	NautilusPlacesSidebar *sidebar;
+
+	sidebar = NAUTILUS_PLACES_SIDEBAR (user_data);
+
+	bookmarks_check_popup_sensitivity (sidebar);
+}
+
 static void
 loading_uri_callback (NautilusWindowInfo *window,
 		      char *location,
@@ -692,19 +778,19 @@ compute_drop_position (GtkTreeView *tree_view,
 				 NautilusPlacesSidebar *sidebar)
 {
 	int bookmarks_index;
-	int num_bookmarks;
+	int num_rows;
 	int row;
 	
 	bookmarks_index = get_bookmark_index (tree_view);
 	
-	num_bookmarks = nautilus_bookmark_list_length (sidebar->bookmarks);
+	num_rows = gtk_tree_model_iter_n_children (GTK_TREE_MODEL (sidebar->filter_model), NULL);
 
 	if (!gtk_tree_view_get_dest_row_at_pos (tree_view,
 					   x,
 					   y,
 					   path,
 					   pos)) {
-		row = bookmarks_index + num_bookmarks - 1;
+		row = num_rows - 1;
 		*path = gtk_tree_path_new_from_indices (row, -1);
 		*pos = GTK_TREE_VIEW_DROP_AFTER;
 		return;
@@ -713,12 +799,19 @@ compute_drop_position (GtkTreeView *tree_view,
 	row = *gtk_tree_path_get_indices (*path);
 	gtk_tree_path_free (*path);
 	
-	if (row < bookmarks_index) {
+	if (row == bookmarks_index - 1) {
+		/* Only allow to drop after separator */
+		*pos = GTK_TREE_VIEW_DROP_AFTER;
+	} else if (row < bookmarks_index) {
 		/* Hardcoded shortcuts can only be dragged into */
 		*pos = GTK_TREE_VIEW_DROP_INTO_OR_BEFORE;
-	}
-	else if (row > bookmarks_index + num_bookmarks - 1) {
-		row = bookmarks_index + num_bookmarks - 1;
+	} else if (row >= num_rows) {
+		row = num_rows - 1;
+		*pos = GTK_TREE_VIEW_DROP_AFTER;
+	} else if (*pos != GTK_TREE_VIEW_DROP_BEFORE &&
+		   sidebar->drag_data_received &&
+		   sidebar->drag_data_info == GTK_TREE_MODEL_ROW) {
+		/* bookmark rows are never dragged into other bookmark rows */
 		*pos = GTK_TREE_VIEW_DROP_AFTER;
 	}
 
@@ -811,7 +904,10 @@ drag_motion_callback (GtkTreeView *tree_view,
 
 	if (pos == GTK_TREE_VIEW_DROP_BEFORE ||
 	    pos == GTK_TREE_VIEW_DROP_AFTER ) {
-		if (can_accept_items_as_bookmarks (sidebar->drag_list)) {
+		if (sidebar->drag_data_received &&
+		    sidebar->drag_data_info == GTK_TREE_MODEL_ROW) {
+			action = GDK_ACTION_MOVE;
+		} else if (can_accept_items_as_bookmarks (sidebar->drag_list)) {
 			action = GDK_ACTION_COPY;
 		} else {
 			action = 0;
@@ -974,26 +1070,25 @@ reorder_bookmarks (NautilusPlacesSidebar *sidebar,
 		   int                   new_position)
 {
 	GtkTreeIter iter;
-	GtkTreePath *path;
 	NautilusBookmark *bookmark;
+	PlaceType type; 
 	int old_position;
-	int bookmarks_index;
 
 	/* Get the selected path */
 
 	if (!get_selected_iter (sidebar, &iter))
 		g_assert_not_reached ();
 
-	path = gtk_tree_model_get_path (GTK_TREE_MODEL (sidebar->store), &iter);
-	old_position = *gtk_tree_path_get_indices (path);
-	gtk_tree_path_free (path);
+	gtk_tree_model_get (GTK_TREE_MODEL (sidebar->store), &iter,
+			    PLACES_SIDEBAR_COLUMN_ROW_TYPE, &type,
+			    PLACES_SIDEBAR_COLUMN_INDEX, &old_position,
+			    -1);
 
-	bookmarks_index = get_bookmark_index (sidebar->tree_view);
-	old_position -= bookmarks_index;
-	if (old_position < 0) {
+	if (type != PLACES_BOOKMARK ||
+	    old_position < 0 ||
+	    old_position >= nautilus_bookmark_list_length (sidebar->bookmarks)) {
 		return;
 	}
-	g_assert (old_position < nautilus_bookmark_list_length (sidebar->bookmarks));
 
 	/* Remove the path from the old position and insert it in the new one */
 
@@ -1022,10 +1117,11 @@ drag_data_received_callback (GtkWidget *widget,
 	GtkTreePath *tree_path;
 	GtkTreeViewDropPosition tree_pos;
 	GtkTreeIter iter;
-	int position, bookmarks_index;
+	int position;
 	GtkTreeModel *model;
 	char *drop_uri;
 	GList *selection_list, *uris;
+	PlaceType type; 
 	gboolean success;
 
 	tree_view = GTK_TREE_VIEW (widget);
@@ -1038,6 +1134,7 @@ drag_data_received_callback (GtkWidget *widget,
 			sidebar->drag_list = NULL;
 		}
 		sidebar->drag_data_received = TRUE;
+		sidebar->drag_data_info = info;
 	}
 
 	g_signal_stop_emission_by_name (widget, "drag-data-received");
@@ -1053,17 +1150,26 @@ drag_data_received_callback (GtkWidget *widget,
 
 	if (tree_pos == GTK_TREE_VIEW_DROP_BEFORE ||
 	    tree_pos == GTK_TREE_VIEW_DROP_AFTER) {
-		/* bookmark addition requested */
-		bookmarks_index = get_bookmark_index (tree_view);
-		position = *gtk_tree_path_get_indices (tree_path);
-	
-		if (tree_pos == GTK_TREE_VIEW_DROP_AFTER) {
+		model = gtk_tree_view_get_model (tree_view);
+
+		if (!gtk_tree_model_get_iter (model, &iter, tree_path)) {
+			goto out;
+		}
+
+		gtk_tree_model_get (model, &iter,
+				    PLACES_SIDEBAR_COLUMN_ROW_TYPE, &type,
+				    PLACES_SIDEBAR_COLUMN_INDEX, &position,
+				    -1);
+
+		if (type != PLACES_SEPARATOR && type != PLACES_BOOKMARK) {
+			goto out;
+		}
+
+		if (type == PLACES_BOOKMARK &&
+		    tree_pos == GTK_TREE_VIEW_DROP_AFTER) {
 			position++;
 		}
-	
-		g_assert (position >= bookmarks_index);
-		position -= bookmarks_index;
-	
+
 		switch (info) {
 		case TEXT_URI_LIST:
 			bookmarks_drop_uris (sidebar, selection_data, position);
@@ -1116,6 +1222,7 @@ drag_data_received_callback (GtkWidget *widget,
 		}
 	}
 
+out:
 	sidebar->drop_occured = FALSE;
 	free_drag_data (sidebar);
 	gtk_drag_finish (context, success, FALSE, time);
@@ -1159,6 +1266,29 @@ bookmarks_popup_menu_detach_cb (GtkWidget *attach_widget,
 	sidebar->popup_menu_empty_trash_item = NULL;
 }
 
+static void
+check_unmount_and_eject (GMount *mount,
+			 GVolume *volume,
+			 GDrive *drive,
+			 gboolean *show_unmount,
+			 gboolean *show_eject)
+{
+	*show_unmount = FALSE;
+	*show_eject = FALSE;
+
+	if (drive != NULL) {
+		*show_eject = g_drive_can_eject (drive);
+	}
+
+	if (volume != NULL) {
+		*show_eject |= g_volume_can_eject (volume);
+	}
+	if (mount != NULL) {
+		*show_unmount = g_mount_can_unmount (mount);
+		*show_eject |= g_mount_can_eject (mount);
+	}
+}
+
 static void
 check_visibility (GMount           *mount,
 		  GVolume          *volume,
@@ -1170,14 +1300,12 @@ check_visibility (GMount           *mount,
 		  gboolean         *show_format)
 {
 	*show_mount = FALSE;
-	*show_unmount = FALSE;
-	*show_eject = FALSE;
 	*show_format = FALSE;
 	*show_rescan = FALSE;
 
-	if (drive != NULL) {
-		*show_eject = g_drive_can_eject (drive);
+	check_unmount_and_eject (mount, volume, drive, show_unmount, show_eject);
 
+	if (drive != NULL) {
 		if (g_drive_is_media_removable (drive) &&
 		    !g_drive_is_media_check_automatic (drive) && 
 		    g_drive_can_poll_for_media (drive))
@@ -1185,15 +1313,9 @@ check_visibility (GMount           *mount,
 	}
 
 	if (volume != NULL) {
-		*show_eject |= g_volume_can_eject (volume);
 		if (mount == NULL)
 			*show_mount = g_volume_can_mount (volume);
 	}
-	
-	if (mount != NULL) {
-		*show_unmount = g_mount_can_unmount (mount);
-		*show_eject |= g_mount_can_eject (mount);
-	}
 
 #ifdef TODO_GIO
 		if (something &&
@@ -1235,6 +1357,12 @@ bookmarks_check_popup_sensitivity (NautilusPlacesSidebar *sidebar)
 				    -1);
 	}
 
+	if (eel_preferences_get_boolean (NAUTILUS_PREFERENCES_ENABLE_TABS)) {
+		gtk_widget_show (sidebar->popup_menu_open_in_new_tab_item);
+	} else {
+		gtk_widget_hide (sidebar->popup_menu_open_in_new_tab_item);
+	}
+
 	gtk_widget_set_sensitive (sidebar->popup_menu_remove_item, (type == PLACES_BOOKMARK));
 	gtk_widget_set_sensitive (sidebar->popup_menu_rename_item, (type == PLACES_BOOKMARK));
 	gtk_widget_set_sensitive (sidebar->popup_menu_empty_trash_item, !nautilus_trash_monitor_is_empty ());
@@ -1269,12 +1397,53 @@ bookmarks_selection_changed_cb (GtkTreeSelection      *selection,
 	bookmarks_check_popup_sensitivity (sidebar);
 }
 
+static void
+volume_mounted_cb (GVolume *volume,
+		   GObject *user_data)
+{
+	GMount *mount;
+	NautilusPlacesSidebar *sidebar;
+	GFile *location;
+
+	sidebar = NAUTILUS_PLACES_SIDEBAR (user_data);
+
+	sidebar->mounting = FALSE;
+
+	mount = g_volume_get_mount (volume);
+	if (mount != NULL) {
+		location = g_mount_get_root (mount);
+
+		if (sidebar->go_to_after_mount_slot != NULL) {
+			if ((sidebar->go_to_after_mount_flags & NAUTILUS_WINDOW_OPEN_FLAG_NEW_WINDOW) == 0) {
+				nautilus_window_slot_info_open_location (sidebar->go_to_after_mount_slot, location,
+									 NAUTILUS_WINDOW_OPEN_ACCORDING_TO_MODE,
+									 sidebar->go_to_after_mount_flags, NULL);
+			} else {
+				NautilusWindow *cur, *new;
+				
+				cur = NAUTILUS_WINDOW (sidebar->window);
+				new = nautilus_application_create_navigation_window (cur->application,
+										     NULL,
+										     gtk_window_get_screen (GTK_WINDOW (cur)));
+				nautilus_window_go_to (new, location);
+			}
+		}
+
+		g_object_unref (G_OBJECT (location));
+		g_object_unref (G_OBJECT (mount));
+	}
+
+	
+	eel_remove_weak_pointer (&(sidebar->go_to_after_mount_slot));
+}
+
 static void
 open_selected_bookmark (NautilusPlacesSidebar *sidebar,
 			GtkTreeModel	      *model,
 			GtkTreePath	      *path,
-			gboolean	      open_in_new_window)
+			NautilusWindowOpenFlags	      flags)
 {
+	NautilusWindowSlotInfo *slot;
 	GtkTreeIter iter;
 	GFile *location;
 	char *uri;
@@ -1287,6 +1456,12 @@ open_selected_bookmark (NautilusPlacesSidebar *sidebar,
 		return;
 	}
 
+	if (flags & NAUTILUS_WINDOW_OPEN_FLAG_NEW_TAB &&
+	    !eel_preferences_get_boolean (NAUTILUS_PREFERENCES_ENABLE_TABS)) {
+		flags &= ~NAUTILUS_WINDOW_OPEN_FLAG_NEW_TAB;
+		flags |= NAUTILUS_WINDOW_OPEN_FLAG_NEW_WINDOW;
+	}
+
 	gtk_tree_model_get (model, &iter, PLACES_SIDEBAR_COLUMN_URI, &uri, -1);
 
 	if (uri != NULL) {
@@ -1295,10 +1470,11 @@ open_selected_bookmark (NautilusPlacesSidebar *sidebar,
 				    sidebar->window, uri);
 		location = g_file_new_for_uri (uri);
 		/* Navigate to the clicked location */
-		if (!open_in_new_window) {
-			nautilus_window_info_open_location (sidebar->window, location,
-							    NAUTILUS_WINDOW_OPEN_ACCORDING_TO_MODE,
-							    0, NULL);
+		if ((flags & NAUTILUS_WINDOW_OPEN_FLAG_NEW_WINDOW) == 0) {
+			slot = nautilus_window_info_get_active_slot (sidebar->window);
+			nautilus_window_slot_info_open_location (slot, location,
+								 NAUTILUS_WINDOW_OPEN_ACCORDING_TO_MODE,
+								 flags, NULL);
 		} else {
 			NautilusWindow *cur, *new;
 			
@@ -1313,9 +1489,22 @@ open_selected_bookmark (NautilusPlacesSidebar *sidebar,
 
 	} else {
 		GVolume *volume;
+		NautilusWindowSlot *slot;
 		gtk_tree_model_get (model, &iter, PLACES_SIDEBAR_COLUMN_VOLUME, &volume, -1);
-		if (volume != NULL) {
-			nautilus_file_operations_mount_volume (NULL, volume, TRUE);
+		if (volume != NULL && !sidebar->mounting) {
+			sidebar->mounting = TRUE;
+
+			g_assert (sidebar->go_to_after_mount_slot == NULL);
+
+			slot = nautilus_window_info_get_active_slot (sidebar->window);
+			sidebar->go_to_after_mount_slot = slot;
+			eel_add_weak_pointer (&(sidebar->go_to_after_mount_slot));
+
+			sidebar->go_to_after_mount_flags = flags;
+
+			nautilus_file_operations_mount_volume_full (NULL, volume, FALSE,
+								    volume_mounted_cb,
+								    G_OBJECT (sidebar));
 			g_object_unref (volume);
 		}
 	}
@@ -1323,7 +1512,7 @@ open_selected_bookmark (NautilusPlacesSidebar *sidebar,
 
 static void
 open_shortcut_from_menu (NautilusPlacesSidebar *sidebar,
-			 gboolean	       open_in_new_window)
+			 NautilusWindowOpenFlags	       flags)
 {
 	GtkTreeModel *model;
 	GtkTreePath *path;
@@ -1331,7 +1520,7 @@ open_shortcut_from_menu (NautilusPlacesSidebar *sidebar,
 	model = gtk_tree_view_get_model (sidebar->tree_view);
 	gtk_tree_view_get_cursor (sidebar->tree_view, &path, NULL);
 
-	open_selected_bookmark (sidebar, model, path, open_in_new_window);
+	open_selected_bookmark (sidebar, model, path, flags);
 
 	gtk_tree_path_free (path);
 }
@@ -1340,14 +1529,21 @@ static void
 open_shortcut_cb (GtkMenuItem		*item,
 		  NautilusPlacesSidebar	*sidebar)
 {
-	open_shortcut_from_menu (sidebar, FALSE);
+	open_shortcut_from_menu (sidebar, 0);
 }
 
 static void
 open_shortcut_in_new_window_cb (GtkMenuItem	      *item,
 				NautilusPlacesSidebar *sidebar)
 {
-	open_shortcut_from_menu (sidebar, TRUE);
+	open_shortcut_from_menu (sidebar, NAUTILUS_WINDOW_OPEN_FLAG_NEW_WINDOW);
+}
+
+static void
+open_shortcut_in_new_tab_cb (GtkMenuItem	      *item,
+				NautilusPlacesSidebar *sidebar)
+{
+	open_shortcut_from_menu (sidebar, NAUTILUS_WINDOW_OPEN_FLAG_NEW_TAB);
 }
 
 /* Rename the selected bookmark */
@@ -1364,7 +1560,7 @@ rename_selected_bookmark (NautilusPlacesSidebar *sidebar)
 		path = gtk_tree_model_get_path (GTK_TREE_MODEL (sidebar->store), &iter);
 		column = gtk_tree_view_get_column (GTK_TREE_VIEW (sidebar->tree_view), 0);
 		renderers = gtk_tree_view_column_get_cell_renderers (column);
-		cell = g_list_nth_data (renderers, 1);
+		cell = g_list_nth_data (renderers, 3);
 		g_list_free (renderers);
 		g_object_set (cell, "editable", TRUE, NULL);
 		gtk_tree_view_set_cursor_on_cell (GTK_TREE_VIEW (sidebar->tree_view),
@@ -1430,41 +1626,45 @@ mount_shortcut_cb (GtkMenuItem           *item,
 			    -1);
 
 	if (volume != NULL) {
-		nautilus_file_operations_mount_volume (NULL, volume, TRUE);
+		nautilus_file_operations_mount_volume (NULL, volume, FALSE);
 		g_object_unref (volume);
 	}
 }
 
+static void
+do_unmount (GMount *mount,
+	    NautilusPlacesSidebar *sidebar)
+{
+	if (mount != NULL) {
+		nautilus_file_operations_unmount_mount (NULL, mount, FALSE, TRUE);
+	}
+}
+
+static void
+do_unmount_selection (NautilusPlacesSidebar *sidebar)
+{
+	GtkTreeIter iter;
+	GMount *mount;
+
+	if (!get_selected_iter (sidebar, &iter)) {
+		return;
+	}
+
+	gtk_tree_model_get (GTK_TREE_MODEL (sidebar->store), &iter,
+			    PLACES_SIDEBAR_COLUMN_MOUNT, &mount,
+			    -1);
+
+	if (mount != NULL) {
+		do_unmount (mount, sidebar);
+		g_object_unref (mount);
+	}
+}
+
 static void
 unmount_shortcut_cb (GtkMenuItem           *item,
 		     NautilusPlacesSidebar *sidebar)
 {
-	GtkTreeIter iter;
-	GMount *mount;
-	GVolume *volume;
-
-	if (!get_selected_iter (sidebar, &iter)) {
-		return;
-	}
-
-	gtk_tree_model_get (GTK_TREE_MODEL (sidebar->store), &iter,
-			    PLACES_SIDEBAR_COLUMN_VOLUME, &volume,
-			    PLACES_SIDEBAR_COLUMN_MOUNT, &mount,
-			    -1);
-
-	if (mount != NULL) {
-		GtkWidget *toplevel;
-		
-		toplevel = gtk_widget_get_toplevel (GTK_WIDGET (sidebar->tree_view));
-		nautilus_file_operations_unmount_mount (GTK_WINDOW (toplevel),
-							mount, FALSE, TRUE);
-	}
-	if (mount != NULL) {
-		g_object_unref (mount);
-	}
-	if (volume != NULL) {
-		g_object_unref (volume);
-	}
+	do_unmount_selection (sidebar);
 }
 
 static void
@@ -1536,6 +1736,21 @@ mount_eject_cb (GObject *source_object,
 	}
 }
 
+static void
+do_eject (GMount *mount,
+	  GVolume *volume,
+	  GDrive *drive,
+	  NautilusPlacesSidebar *sidebar)
+{
+	if (mount != NULL) {
+		g_mount_eject (mount, 0, NULL, mount_eject_cb, NULL);
+	} else if (volume != NULL) {
+		g_volume_eject (volume, 0, NULL, volume_eject_cb, NULL);
+	} else if (drive != NULL) {
+		g_drive_eject (drive, 0, NULL, drive_eject_cb, NULL);
+	}
+}
+
 static void
 eject_shortcut_cb (GtkMenuItem           *item,
 		   NautilusPlacesSidebar *sidebar)
@@ -1555,12 +1770,46 @@ eject_shortcut_cb (GtkMenuItem           *item,
 			    PLACES_SIDEBAR_COLUMN_DRIVE, &drive,
 			    -1);
 
-	if (mount != NULL) {
-		g_mount_eject (mount, 0, NULL, mount_eject_cb, NULL);
-	} else if (volume != NULL) {
-		g_volume_eject (volume, 0, NULL, volume_eject_cb, NULL);
-	} else if (drive != NULL) {
-		g_drive_eject (drive, 0, NULL, drive_eject_cb, NULL);
+	do_eject (mount, volume, drive, sidebar);
+}
+
+static gboolean
+eject_or_unmount_bookmark (NautilusPlacesSidebar *sidebar,
+			   GtkTreePath *path)
+{
+	GtkTreeModel *model;
+	GtkTreeIter iter;
+	gboolean can_unmount, can_eject;
+	GMount *mount;
+	GVolume *volume;
+	GDrive *drive;
+	gboolean ret;
+
+	model = GTK_TREE_MODEL (sidebar->filter_model);
+
+	if (!path) {
+		return FALSE;
+	}
+	if (!gtk_tree_model_get_iter (model, &iter, path)) {
+		return FALSE;
+	}
+
+	gtk_tree_model_get (model, &iter,
+			    PLACES_SIDEBAR_COLUMN_MOUNT, &mount,
+			    PLACES_SIDEBAR_COLUMN_VOLUME, &volume,
+			    PLACES_SIDEBAR_COLUMN_DRIVE, &drive,
+			    -1);
+
+	ret = FALSE;
+
+	check_unmount_and_eject (mount, volume, drive, &can_unmount, &can_eject);
+	/* if we can eject, it has priority over unmount */
+	if (can_eject) {
+		do_eject (mount, volume, drive, sidebar);
+		ret = TRUE;
+	} else if (can_unmount) {
+		do_unmount (mount, sidebar);
+		ret = TRUE;
 	}
 
 	if (mount != NULL)
@@ -1569,6 +1818,31 @@ eject_shortcut_cb (GtkMenuItem           *item,
 		g_object_unref (volume);
 	if (drive != NULL)
 		g_object_unref (drive);
+
+	return ret;
+}
+
+static gboolean
+eject_or_unmount_selection (NautilusPlacesSidebar *sidebar)
+{
+	GtkTreeIter iter;
+	GtkTreePath *path;
+	gboolean ret;
+
+	if (!get_selected_iter (sidebar, &iter)) {
+		return FALSE;
+	}
+
+	path = gtk_tree_model_get_path (GTK_TREE_MODEL (sidebar->store), &iter);
+	if (path == NULL) {
+		return FALSE;
+	}
+
+	ret = eject_or_unmount_bookmark (sidebar, path);
+
+	gtk_tree_path_free (path);
+
+	return ret;
 }
 
 static void
@@ -1640,6 +1914,11 @@ bookmarks_key_press_event_cb (GtkWidget             *widget,
 
   modifiers = gtk_accelerator_get_default_mod_mask ();
 
+  if (event->keyval == GDK_Down &&
+      (event->state & modifiers) == GDK_MOD1_MASK) {
+      return eject_or_unmount_selection (sidebar);
+  }
+
   if ((event->keyval == GDK_BackSpace
       || event->keyval == GDK_Delete
       || event->keyval == GDK_KP_Delete)
@@ -1680,12 +1959,21 @@ bookmarks_build_popup_menu (NautilusPlacesSidebar *sidebar)
 	gtk_widget_show (item);
 	gtk_menu_shell_append (GTK_MENU_SHELL (sidebar->popup_menu), item);
 
+	item = gtk_menu_item_new_with_mnemonic (_("Open in New _Tab"));
+	sidebar->popup_menu_open_in_new_tab_item = item;
+	g_signal_connect (item, "activate",
+			  G_CALLBACK (open_shortcut_in_new_tab_cb), sidebar);
+	gtk_widget_show (item);
+	gtk_menu_shell_append (GTK_MENU_SHELL (sidebar->popup_menu), item);
+
 	item = gtk_menu_item_new_with_mnemonic (_("Open in New _Window"));
 	g_signal_connect (item, "activate",
 			  G_CALLBACK (open_shortcut_in_new_window_cb), sidebar);
 	gtk_widget_show (item);
 	gtk_menu_shell_append (GTK_MENU_SHELL (sidebar->popup_menu), item);
 
+	eel_gtk_menu_append_separator (GTK_MENU (sidebar->popup_menu));
+
 	item = gtk_image_menu_item_new_with_label (_("Remove"));
 	sidebar->popup_menu_remove_item = item;
 	gtk_image_menu_item_set_image (GTK_IMAGE_MENU_ITEM (item),
@@ -1781,16 +2069,54 @@ bookmarks_popup_menu_cb (GtkWidget *widget,
 }
 
 /* Callback used when a button is pressed on the shortcuts list.  
- * We trap button 3 to bring up a popup menu.
+ * We trap button 3 to bring up a popup menu, and button 2 to
+ * open in a new tab.
  */
 static gboolean
 bookmarks_button_press_event_cb (GtkWidget             *widget,
 				 GdkEventButton        *event,
 				 NautilusPlacesSidebar *sidebar)
 {
+	GtkTreePath *path;
+	gboolean ret;
+
+	if (event->type != GDK_BUTTON_PRESS) {
+		/* ignore multiple clicks */
+		return TRUE;
+	}
+
+	if (clicked_eject_button (sidebar, &path)) {
+		ret = eject_or_unmount_bookmark (sidebar, path);
+		gtk_tree_path_free (path);
+		return ret;
+	}
+
 	if (event->button == 3) {
 		bookmarks_popup_menu (sidebar, event);
+	} else if (event->button == 2) {
+		GtkTreeModel *model;
+		GtkTreePath *path;
+		GtkTreeView *tree_view;
+
+		tree_view = GTK_TREE_VIEW (widget);
+		g_assert (tree_view == sidebar->tree_view);
+
+		model = gtk_tree_view_get_model (tree_view);
+
+		gtk_tree_view_get_path_at_pos (tree_view, (int) event->x, (int) event->y, 
+					       &path, NULL, NULL, NULL);
+
+		open_selected_bookmark (sidebar, model, path,
+					event->state & GDK_CONTROL_MASK ?
+					NAUTILUS_WINDOW_OPEN_FLAG_NEW_WINDOW :
+					NAUTILUS_WINDOW_OPEN_FLAG_NEW_TAB);
+
+		if (path != NULL) {
+			gtk_tree_path_free (path);
+			return TRUE;
+		}
 	}
+
 	return FALSE;
 }
 
@@ -1865,18 +2191,52 @@ nautilus_places_sidebar_init (NautilusPlacesSidebar *sidebar)
 	gtk_tree_view_set_headers_visible (tree_view, FALSE);
 
 	col = GTK_TREE_VIEW_COLUMN (gtk_tree_view_column_new ());
-	
+
 	cell = gtk_cell_renderer_pixbuf_new ();
+	sidebar->icon_cell_renderer = cell;
 	gtk_tree_view_column_pack_start (col, cell, FALSE);
 	gtk_tree_view_column_set_attributes (col, cell,
 					     "pixbuf", PLACES_SIDEBAR_COLUMN_ICON,
 					     NULL);
+
 	
 	cell = gtk_cell_renderer_text_new ();
+	sidebar->eject_text_cell_renderer = cell;
 	gtk_tree_view_column_pack_start (col, cell, TRUE);
 	gtk_tree_view_column_set_attributes (col, cell,
 					     "text", PLACES_SIDEBAR_COLUMN_NAME,
+					     "visible", PLACES_SIDEBAR_COLUMN_EJECT,
 					     NULL);
+	g_object_set (cell,
+		      "ellipsize", PANGO_ELLIPSIZE_END,
+		      "ellipsize-set", TRUE,
+		      NULL);
+
+
+	cell = gtk_cell_renderer_pixbuf_new ();
+	g_object_set (cell,
+		      "mode", GTK_CELL_RENDERER_MODE_ACTIVATABLE,
+		      "icon-name", "media-eject",
+		      "stock-size", GTK_ICON_SIZE_MENU,
+		      "xpad", EJECT_BUTTON_XPAD,
+		      NULL);
+	gtk_tree_view_column_pack_start (col, cell, FALSE);
+	gtk_tree_view_column_set_attributes (col, cell,
+					     "visible", PLACES_SIDEBAR_COLUMN_EJECT,
+					     NULL);
+
+	cell = gtk_cell_renderer_text_new ();
+	gtk_tree_view_column_pack_start (col, cell, TRUE);
+	g_object_set (G_OBJECT (cell), "editable", FALSE, NULL);
+	gtk_tree_view_column_set_attributes (col, cell,
+					     "text", PLACES_SIDEBAR_COLUMN_NAME,
+					     "visible", PLACES_SIDEBAR_COLUMN_NO_EJECT,
+					     "editable-set", PLACES_SIDEBAR_COLUMN_BOOKMARK,
+					     NULL);
+	g_object_set (cell,
+		      "ellipsize", PANGO_ELLIPSIZE_END,
+		      "ellipsize-set", TRUE,
+		      NULL);
 
 	g_signal_connect (cell, "edited", 
 			  G_CALLBACK (bookmarks_edited), sidebar);
@@ -1888,9 +2248,10 @@ nautilus_places_sidebar_init (NautilusPlacesSidebar *sidebar)
 					      NULL,
 					      NULL);
 
-	gtk_tree_view_column_set_fixed_width (col, NAUTILUS_ICON_SIZE_SMALLER);
+	/* this is required to align the eject buttons to the right */
+	gtk_tree_view_column_set_max_width (GTK_TREE_VIEW_COLUMN (col), NAUTILUS_ICON_SIZE_SMALLER);
 	gtk_tree_view_append_column (tree_view, col);
-	
+
 	sidebar->store = gtk_list_store_new (PLACES_SIDEBAR_COLUMN_COUNT,
 					     G_TYPE_INT, 
 					     G_TYPE_STRING,
@@ -1899,7 +2260,11 @@ nautilus_places_sidebar_init (NautilusPlacesSidebar *sidebar)
 					     G_TYPE_MOUNT,
 					     G_TYPE_STRING,
 					     GDK_TYPE_PIXBUF,
-					     G_TYPE_INT
+					     G_TYPE_INT,
+					     G_TYPE_BOOLEAN,
+					     G_TYPE_BOOLEAN,
+					     G_TYPE_BOOLEAN,
+					     G_TYPE_BOOLEAN
 					     );
 	sidebar->filter_model = nautilus_shortcuts_model_filter_new (sidebar,
 								     GTK_TREE_MODEL (sidebar->store),
@@ -1950,17 +2315,19 @@ nautilus_places_sidebar_init (NautilusPlacesSidebar *sidebar)
 	g_signal_connect (tree_view, "button-press-event",
 			  G_CALLBACK (bookmarks_button_press_event_cb), sidebar);
 
-	eel_preferences_add_callback_while_alive (NAUTILUS_PREFERENCES_CLICK_POLICY,
-						  click_policy_changed_callback,
-						  sidebar,
-						  G_OBJECT (sidebar));
-	update_click_policy (sidebar);
+	eel_gtk_tree_view_set_activate_on_single_click (sidebar->tree_view,
+							TRUE);
 
 	eel_preferences_add_callback_while_alive (NAUTILUS_PREFERENCES_DESKTOP_IS_HOME_DIR,
 						  desktop_location_changed_callback,
 						  sidebar,
 						  G_OBJECT (sidebar));
 
+	eel_preferences_add_callback_while_alive (NAUTILUS_PREFERENCES_ENABLE_TABS,
+						  enable_tabs_changed_callback,
+						  sidebar,
+						  G_OBJECT (sidebar));
+
 	g_signal_connect_object (nautilus_trash_monitor_get (),
 				 "trash_state_changed",
 				 G_CALLBACK (trash_state_changed_cb),
@@ -1992,6 +2359,8 @@ nautilus_places_sidebar_dispose (GObject *object)
 		sidebar->volume_monitor = NULL;
 	}
 
+	eel_remove_weak_pointer (&(sidebar->go_to_after_mount_slot));
+
 	G_OBJECT_CLASS (nautilus_places_sidebar_parent_class)->dispose (object);
 }
 
@@ -2047,11 +2416,15 @@ nautilus_places_sidebar_iface_init (NautilusSidebarIface *iface)
 static void
 nautilus_places_sidebar_set_parent_window (NautilusPlacesSidebar *sidebar,
 					   NautilusWindowInfo *window)
-{	
+{
+	NautilusWindowSlotInfo *slot;
+
 	sidebar->window = window;
-	
+
+	slot = nautilus_window_info_get_active_slot (window);
+
 	sidebar->bookmarks = nautilus_window_info_get_bookmark_list (window);
-	sidebar->uri = nautilus_window_info_get_current_location (window);
+	sidebar->uri = nautilus_window_slot_info_get_current_location (slot);
 
 	g_signal_connect_object (sidebar->bookmarks, "contents_changed",
 				 G_CALLBACK (update_places),
